#!/usr/bin/env python3

# Lost Treasure
# Based on "Stimulating Simulations, 2nd edition (Atari Version)"
# by C. William Engel, 1977, 1979.
# https://archive.org/details/ataribooks-stimulating-simulations-atari-version

# Copyright © 2019 by Mark Damon Hughes. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os, sys
import math
import random

kMapSize = 11
kGrid_Clear = '.'	# 0
kGrid_Woods = 'W'	# 1
kGrid_Mountain = '^'	# 2
kGrid_Cave = 'C'	# 3
kGrid_Bluff = 'B'	# 4
kGrid_Oak = 'O'		# 5
kGrid_Treasure = '$'	# 6
kGrid_Water = '~'

WORLD = None

INTRO_TEXT = """\
 _           _     _____
| |   ___ __| |_  |_   _| _ ___ __ _ ____  _ _ _ ___
| |__/ _ (_-<  _|   | || '_/ -_) _` (_-< || | '_/ -_)
|____\___/__/\__|   |_||_| \___\__,_/__/\_,_|_| \___|

original game by C. William Engel in Stimulating Simulations, 1977
adapted by Mark Damon Hughes, 2019
version 1.0

You have landed somewhere on an island with a treasure map, but you are lost.
Your compass is not very accurate, so you will not always travel in a straight
line. Use the landmarks to find your way. Beware the shark-infested waters.
Find the treasure as quickly as possible!

Type N, E, S, W to move, M to examine the map.
"""

MAPKEY_TEXT = """Key:
. = Clear		W = Woods		^ = Mountains
C = Cave		B = Bluff		O = Oak Tree
$ = Treasure		~ = Ocean
"""

#_______________________________________

def die(s):
	return random.randint(1,s)

def pointAdd(a, b):
	return ( (a[0]+b[0], a[1]+b[1]) )

def pointDist(a, b):
	return math.sqrt( (a[0]-b[0])*(a[0]-b[0]) + (a[1]-b[1])*(a[1]-b[1]) )

#_______________________________________

class Player:
	def __init__(self):
		self.loc = (0,0)
		while True:
			self.loc = [die(kMapSize-2), die(kMapSize-2)]
			if pointDist(self.loc, WORLD.map.treasureLoc) > 2:
				break
		self.turn = 0
		self.sharkChance = 10
		self.wanderChance = 10

class Island:
	def __init__(self):
		self.map = []
		for y in range(kMapSize):
			row = []
			for x in range(kMapSize):
				if x == 0 or x == kMapSize-1 or y == 0 or y == kMapSize-1:
					g = kGrid_Water
				else:
					g = kGrid_Clear
				row.append(g)
			self.map.append(row)
		for pt in ( (3,2), (5,3), (9,3), (1,4), (2,7), (8,8) ):
			 self.setGridAt(pt, kGrid_Woods)
		for pt in ( (2,1), (7,3), (2,5), (8,6), (3,8), (6,8) ):
			self.setGridAt(pt, kGrid_Mountain)
		self.setGridAt((8,1), kGrid_Cave)
		self.setGridAt((1,6), kGrid_Bluff)
		self.setGridAt((6,9), kGrid_Oak)
		self.treasureLoc = (5,5)
		self.setGridAt(self.treasureLoc, kGrid_Treasure)

	def printMap(self):
		text = ""
		for y in range(kMapSize):
			for x in range(kMapSize):
				text += self.gridAt((x,y)) + " "
			text += "\n"
		print(text)

	def gridAt(self, pt):
		return self.map[pt[1]][pt[0]]

	def inBounds(self, pt):
		return pt[0]>=0 and pt[0]<kMapSize and pt[1]>=0 and pt[1]<kMapSize

	def setGridAt(self, pt, g):
		self.map[pt[1]][pt[0]] = g

class Game:
	def __init__(self):
		global WORLD
		WORLD = self
		self.map = Island()
		self.player = Player()

	def run(self):
		print(INTRO_TEXT)
		input("[Press Enter to show map]")
		self.showMap()

		self.playing = True
		while self.playing:
			self.player.turn += 1
			self.look()
			if not self.playing: break

			line = input("Move (N, E, S, W, M)? ")
			line = line.strip().upper()
			if line == "N":
				self.move((0,-1))
			elif line == "E":
				self.move((1,0))
			elif line == "S":
				self.move((0,1))
			elif line == "W":
				self.move((-1,0))
			elif line == "M":
				self.showMap()
			else:
				print("What?")
		print("*** Game Over ***")

	def look(self):
		text = "Turn %d. " % (self.player.turn,)
		g = self.map.gridAt(self.player.loc)
		if g == kGrid_Clear:
			text += "You are in the clear."
		elif g == kGrid_Woods:
			text += "You are in the woods."
		elif g == kGrid_Mountain:
			text += "You are in the mountains."
		elif g == kGrid_Cave:
			text += "You are in a dark cave."
		elif g == kGrid_Bluff:
			text += "You are on a high bluff."
		elif g == kGrid_Oak:
			text += "You are under an old oak tree."
		elif g == kGrid_Treasure:
			text += "You have found the treasure!"
			text += "\n*** YOU WIN! ***"
			self.playing = False
		elif g == kGrid_Water:
			text += "in the ocean!"
			if die(100) <= self.player.sharkChance:
				text += "\nSharks find you and tear you apart."
				text += "\n*** YOU DIE! ***"
				self.playing = False
			else:
				self.player.sharkChance *= 1.5
				if self.player.sharkChance <= 25:
					text += "\nThe sharks are distant..."
				elif self.player.sharkChance <= 50:
					text += "\nThe sharks are getting closer..."
				else:
					text += "\nThe sharks are swarming around you!"
		else:
			text += "You are in unknown grid %s!" % (g,)
		print(text)

	def move(self, delta):
		wander = die(100)
		if delta[0] == 0:
			if wander <= self.player.wanderChance // 2:
				delta = pointAdd(delta, (-1,0))
			elif wander <= self.player.wanderChance:
				delta = pointAdd(delta, (1,0))
		elif delta[1] == 0:
			if wander <= self.player.wanderChance // 2:
				delta = pointAdd(delta, (0,-1))
			elif wander <= self.player.wanderChance:
				delta = pointAdd(delta, (0,1))
		# move
		pt2 = pointAdd(self.player.loc, delta)
		if self.map.inBounds(pt2):
			self.player.loc = pt2
			return True
		else:
			print("You can't go that way!'")
			return False

	def showMap(self):
		print("Map:")
		self.map.printMap()
		print(MAPKEY_TEXT)

if __name__ == "__main__":
	try:
		Game().run()
	except (KeyboardInterrupt, EOFError):
		print("\nGoodbye!")
